<?php

namespace Drupal\password_notification\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Delete Password Changed History.
 */
class PasswordChangedHistoryDeleteForm extends FormBase {
  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;
  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'password_changed_history_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $database, MessengerInterface $messenger) {
    $this->database = $database;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $total_records = 'No Result.';
    $record_exist = FALSE;

    // Get Records Count.
    $get_result_count = $this->database->select('password_changed_history', 'p')
      ->fields('p', ['id'])
      ->countQuery()
      ->execute()
      ->fetchField();
    if ($get_result_count) {
      $total_records = 'Total Records: ' . $get_result_count;
      $record_exist = TRUE;
    }
    $form['records_count'] = [
      '#markup' => '<div>' . $total_records . '</div>',
    ];
    if ($record_exist == TRUE) {
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => t('Delete Records'),
      ];

      $password_history_config_route = Url::fromRoute('password_notification.config');
      $password_history_config_link = Link::fromTextAndUrl('this page', $password_history_config_route)->toString();
      $form['help_text'] = [
        '#markup' => '<div>Note: By default, the delete records limit is 1000. To update the limit, go to ' . $password_history_config_link . ' and update the limit.</div>',
      ];

    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $batch = [
      'title' => t('Batch Operation'),
      'init_message' => t('Batch Operation starting...'),
      'progress_message' => t('Processed @current out of @total.'),
      'error_message' => t('Fix has encountered an error.'),
      'operations' => [],
      'finished' => [$this, 'finishedCallback'],
    ];

    $config = \Drupal::config('password_notification.settings');
    $limit = $config->get('delete_records_limit');
    $query = $this->database->select('password_changed_history', 'p');
    $query->fields('p', ['id']);
    $query->range(0, $limit);
    $query->orderBy('p.created', 'ASC');
    $ids = $query->execute()->fetchCol();
    foreach ($ids as $result) {
      $batch['operations'][] = [
        [$this, 'batchFunction'],
        [$result],
      ];
    }

    batch_set($batch);
  }

  /**
   * Batch Operation Callback.
   */
  public function batchFunction($id) {

    $this->database->delete('password_changed_history')
      ->condition('id', $id, '=')
      ->execute();
  }

  /**
   * Batch Operation Finished Callback..
   */
  public function finishedCallback() {
    $this->messenger->addMessage($this->t('Delete Process Completed.'));
  }

}
